searchNodes=[{"ref":"Certbot.html","title":"Certbot","type":"module","doc":"Provide certificates for your Phoenix or Plug app using Letsencrypt. This package should for now be considered a POC. Not everything is implemented at the moment, most notably, certificate renewal. You can also set your own Certificate Provider for your own functionality, or to provide different certificates for different hostnames. Installation The package can be installed by adding certbot to your list of dependencies in mix.exs: def deps do [ {:certbot, &quot;~&gt; 0.5.0&quot;} ] end Setting up Letsencrypt with Phoenix From then on there are a few steps, we need to setup a certbot client, a store for the certificates and a store for Acme challenges. Furthermore we need to setup Certbot.Acme.Plug to verify Acme challenges over http. Certbot client First, a certbot client is needed. We use a self generated private key to build into a JWK. If you have a Phoenix project, this can be generated with mix phx.gen.cert Furthermore, we set an AcmeCertificateProvider defmodule Myapp.CertbotClient do @jwk &quot;priv/cert/selfsigned_key.pem&quot; |&gt; File.read!() |&gt; JOSE.JWK.from_pem() |&gt; JOSE.JWK.to_map() use Certbot, certificate_provider: Myapp.AcmeCertificateProvider, jwk: @jwk, email: &quot;mailto:test@example.com&quot; end defmodule Myapp.AcmeCertificateProvider do use Certbot.Provider.Acme, challenge_store: Certbot.Acme.ChallengeStore.Default, certificate_store: Certbot.CertificateStore.Default, acme_client: Myapp.CertbotClient end The Myapp.CertbotClient doubles as an Acme client, and therefore needs to be added to the supervision tree of your application. We use, the default challenge/certificate stores of the package, they also need to be added your application supervision tree. Note, there are downsides to the stores, see their docs for more info. Your supervision tree will look something like this in a Phoenix project # application.ex children = [ # Start the Ecto repository Myapp.Repo, # Start the endpoint when the application starts MyappWeb.Endpoint, Myapp.CertbotClient, Certbot.Acme.ChallengeStore.Default, Certbot.CertificateStore.Default ] In your endpoint.ex you should add Certbot.Acme.Plug, with the same challenge store and jwk. It should be added before the router, and before Plug.SSL if force SSL redirects are turned on. # endpoint.ex @jwk &quot;priv/cert/selfsigned_key.pem&quot; |&gt; File.read!() |&gt; JOSE.JWK.from_pem() |&gt; JOSE.JWK.to_map() plug Certbot.Acme.Plug, challenge_store: Certbot.Acme.ChallengeStore.Default, jwk: @jwk plug MyappWeb.Router As a last step we need configure the https endpoint to dynamically return certificates. config :myapp, MyappWeb.Endpoint, http: [port: 6000], https: [ cipher_suite: :strong, port: 6001, sni_fun: &amp;Myapp.CertbotClient.sni_fun/1 #Set the sni_fun ], This tells cowboy to call sni_fun/1 with the hostname of the request. This function will ask the certificate provider for a certificate. The certificate provider will return one, or first request one from Letsencrypt and then return it. FAQ Is this tested in production? No, be careful Can I test this against a non-production acme server Yes, you need to set the :server to https://acme-staging.api.letsencrypt.org/ Does it do certificate renewal? Not yet, should not be really hard to do. Every now and then a sweep of the certificate store to check for certificates that are about to expire, and renew a certificate for them. How can I test this locally? You need to make sure port 80 is available for the Acme server to request with the token verification call. You&#39;ll need to map port 80 to the https port you configured your endpoint to. Are multiple account keys supported? No, not yet. But willing to accept PR&#39;s. How are multiple concurrent requests handled with certification requests? Nothing is done, ideally some kind of lock is placed so requests after the first one will wait till a certificate is retrieved and then use this certificate. Nothing of the kind is done. What happens if I request too many certificates? You&#39;ll be ratelimited by Letsencrypt I am debugging but don&#39;t see errors appearing? Because everything happens as a result of calling the sni_fun/1 callback, this is at such a level that many errors don&#39;t seem to appear. What version of the Acme protocol is used? Acme V1 is used, many thanks to this package: https://github.com/sikanhe/acme Are alternative challenge methods (dns-01, tls-sni-01,tls-alpn-01)? No, tls-alpn-01 is currently not supported by the Acme client but would be interesting as it would make it unnecessary te expose port 80. tls-sni-01 is not secure and dns-01 is out of scope as of now. Errors [error] %Certbot.Error{detail: &quot;JWS has invalid anti-replay nonce twT0up7DWSrbe163DiRuKnPwd4ZpyXVER0p-COl1vAA&quot;, status: 400, type: &quot;urn:acme:error:badNonce&quot;} This is not handled yet, the nonce should be refreshed and then the request repeated. [error] Certbot.Acme.Client Certbot.Acme.Client received unexpected message in handle_info/2: {:ssl_closed, {:sslsocket, {:gen_tcp, #Port&lt;0.76&gt;, :tls_connection, :undefined}, [#PID&lt;0.851.0&gt;, #PID&lt;0.850.0&gt;]}} Don&#39;t know why this happens... Documentation Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/certbot."},{"ref":"Certbot.html#sni_fun/2","title":"Certbot.sni_fun/2","type":"function","doc":""},{"ref":"Certbot.html#start_link/2","title":"Certbot.start_link/2","type":"function","doc":""},{"ref":"Certbot.Acme.Authorization.html","title":"Certbot.Acme.Authorization","type":"module","doc":"Module for handling an Acme.Authorization response, converted from an %Acme.Authorization{} struct"},{"ref":"Certbot.Acme.Authorization.html#fetch_challenge/2","title":"Certbot.Acme.Authorization.fetch_challenge/2","type":"function","doc":"Fetch specific challenge by type from an authorization struct Possible types: http-01 dns-01 tls-sni-01 Example iex&gt; Certbot.Acme.Authorization.fetch_challenge(%Certbot.Acme.Authorization{challenges: []}, &quot;http-01&quot;) nil iex&gt; challenge = %Certbot.Acme.Challenge{type: &quot;http-01&quot;} iex&gt; Certbot.Acme.Authorization.fetch_challenge(%Certbot.Acme.Authorization{challenges: [challenge]}, &quot;http-01&quot;) %Certbot.Acme.Challenge{ status: nil, token: nil, type: &quot;http-01&quot;, uri: nil }"},{"ref":"Certbot.Acme.Authorization.html#from_map/1","title":"Certbot.Acme.Authorization.from_map/1","type":"function","doc":""},{"ref":"Certbot.Acme.Authorization.html#t:t/0","title":"Certbot.Acme.Authorization.t/0","type":"type","doc":""},{"ref":"Certbot.Acme.Challenge.html","title":"Certbot.Acme.Challenge","type":"module","doc":"Struct with utility functions for dealing with Acme Challenges"},{"ref":"Certbot.Acme.Challenge.html#authorization/2","title":"Certbot.Acme.Challenge.authorization/2","type":"function","doc":"Return authorization token given a challenge and the jwk used to generate it Example iex&gt; jwk = &quot;test/fixtures/selfsigned_key.pem&quot; |&gt; File.read!() |&gt; JOSE.JWK.from_pem() |&gt; JOSE.JWK.to_map() iex&gt; Certbot.Acme.Challenge.authorization(&quot;some_token&quot;, jwk) &quot;some_token.v5Co8pJG2fo_hBcdhEzpj_DSEcev76KkbFQkJRiu-Cg&quot;"},{"ref":"Certbot.Acme.Challenge.html#from_struct/1","title":"Certbot.Acme.Challenge.from_struct/1","type":"function","doc":"Convert structs of the same shape, to a Certbot.Acme.Challenge struct. Example iex&gt; Certbot.Acme.Challenge.from_struct(%Acme.Challenge{type: &quot;http-01&quot;}) %Certbot.Acme.Challenge{ status: nil, token: nil, type: &quot;http-01&quot;, uri: nil }"},{"ref":"Certbot.Acme.Challenge.html#t:t/0","title":"Certbot.Acme.Challenge.t/0","type":"type","doc":""},{"ref":"Certbot.Acme.ChallengeStore.html","title":"Certbot.Acme.ChallengeStore","type":"behaviour","doc":"Behaviour for storing and finding challenges Implement this behaviour for a challengestore based on another persistence mechanism."},{"ref":"Certbot.Acme.ChallengeStore.html#c:find_by_token/1","title":"Certbot.Acme.ChallengeStore.find_by_token/1","type":"callback","doc":""},{"ref":"Certbot.Acme.ChallengeStore.html#c:insert/1","title":"Certbot.Acme.ChallengeStore.insert/1","type":"callback","doc":""},{"ref":"Certbot.Acme.ChallengeStore.Default.html","title":"Certbot.Acme.ChallengeStore.Default","type":"module","doc":"Default Acme.ChallengeStore Used to store challenges provided by the Acme server. This is a simple genserver, if you have multiple servers running the challenges won&#39;t be distributed among them so the challenges will fail. To counter this you&#39;ll need to reimplement this store with another one, e.g. based on mnesia or redis, and should implement the Certbot.Acme.ChallengeStore behaviour"},{"ref":"Certbot.Acme.ChallengeStore.Default.html#child_spec/1","title":"Certbot.Acme.ChallengeStore.Default.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Certbot.Acme.ChallengeStore.Default.html#start_link/1","title":"Certbot.Acme.ChallengeStore.Default.start_link/1","type":"function","doc":""},{"ref":"Certbot.Acme.Plug.html","title":"Certbot.Acme.Plug","type":"module","doc":"Plug used to intercept challenge verification calls on the request path /.well-known/acme-challenge/&lt;token&gt;. The plug can be placed early in the pipeline. When using Phoenix, it should be placed before your router in your endpoint.ex. If you plan on redirecting http to https using Plug.SSL, place it after this plug. Certbot.Acme.Plug needs to work over http. It requires two options. :challenge_store -- The challenge store used, so when a verication call comes in, it can check whether it knows the token. It needs to be the same store where the Certbot.Provider.Acme provider stores the challenges. :jwk -- A jwk map, see below for an example on how to generate one from a private key. Example @jwk &quot;priv/cert/selfsigned_key.pem&quot; |&gt; File.read!() |&gt; JOSE.JWK.from_pem() |&gt; JOSE.JWK.to_map() plug Certbot.Acme.Plug, challenge_store: Certbot.ChallengeStore.Default, jwk: @jwk"},{"ref":"Certbot.Acme.Plug.html#call/2","title":"Certbot.Acme.Plug.call/2","type":"function","doc":""},{"ref":"Certbot.Acme.Plug.html#init/1","title":"Certbot.Acme.Plug.init/1","type":"function","doc":""},{"ref":"Certbot.Certificate.html","title":"Certbot.Certificate","type":"module","doc":"The module provides utility functions to deal with the serial and validity timestamps of a certificate as well as being a struct to store certificate/keys"},{"ref":"Certbot.Certificate.html#build/2","title":"Certbot.Certificate.build/2","type":"function","doc":"Build struct to store a certificate in der format and its private key For example, to generate a cert_file = File.read!(&quot;priv/cert/selfsigned.pem&quot;) key_file = File.read!(&quot;priv/cert/selfsigned_key.pem&quot;) [certificate] = :public_key.pem_decode(cert_file) cert = :public_key.pem_entry_decode(certificate) cert = :public_key.der_encode(:Certificate, cert) [key] = :public_key.pem_decode(key_file) key = :public_key.pem_entry_decode(key) der_key = :public_key.der_encode(:RSAPrivateKey, key) Certbot.Certificate.build(cert, {:RSAPrivateKey, der_key})"},{"ref":"Certbot.Certificate.html#hex_serial/1","title":"Certbot.Certificate.hex_serial/1","type":"function","doc":"Get hexadecimal serial number of the certicate This is what is visible when inspecting a certificate in the browser Example iex&gt; Certbot.Certificate.hex_serial(build_certificate()) &quot;FC100FFC200BF62F&quot;"},{"ref":"Certbot.Certificate.html#serial/1","title":"Certbot.Certificate.serial/1","type":"function","doc":"Get integer serial number of the certicate iex&gt; Certbot.Certificate.serial(build_certificate()) 18163034872729040431"},{"ref":"Certbot.Certificate.html#valid_from/1","title":"Certbot.Certificate.valid_from/1","type":"function","doc":"Get DateTime of the date the certificate was given out Example iex&gt; Certbot.Certificate.valid_from(build_certificate()) ~U[2019-07-09 00:00:00Z]"},{"ref":"Certbot.Certificate.html#valid_until/1","title":"Certbot.Certificate.valid_until/1","type":"function","doc":"Get DateTime of the date the certificate will be valid until Example iex&gt; Certbot.Certificate.valid_until(build_certificate()) ~U[2020-07-09 00:00:00Z]"},{"ref":"Certbot.Certificate.html#t:t/0","title":"Certbot.Certificate.t/0","type":"type","doc":""},{"ref":"Certbot.CertificateStore.html","title":"Certbot.CertificateStore","type":"behaviour","doc":"Stores and look ups certificates by hostname Behaviour that can be reimplemented for other storage mechanisms, e.g. redis, database or mnesia"},{"ref":"Certbot.CertificateStore.html#c:find_certificate/1","title":"Certbot.CertificateStore.find_certificate/1","type":"callback","doc":""},{"ref":"Certbot.CertificateStore.html#c:insert/2","title":"Certbot.CertificateStore.insert/2","type":"callback","doc":""},{"ref":"Certbot.CertificateStore.Default.html","title":"Certbot.CertificateStore.Default","type":"module","doc":"Default store for certificates. Stores certificates in an ets table. This store won&#39;t work when you have multiple servers as the certificates will only be stored on one server."},{"ref":"Certbot.CertificateStore.Default.html#child_spec/1","title":"Certbot.CertificateStore.Default.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Certbot.CertificateStore.Default.html#start_link/1","title":"Certbot.CertificateStore.Default.start_link/1","type":"function","doc":""},{"ref":"Certbot.Config.html","title":"Certbot.Config","type":"module","doc":"Configuration for Certbot These are the options supplied to use Certbot, server: &quot;&quot; :jwk -- jwk for the account private key for the Acme client :email -- email for the account for the Acme client e.g. mailto:test@example.com :certificate_provider -- module that will responsible for getting certificates from the store, or requesting new certifictes :server -- Server implementing the Acme protocol, most likely that you want the staging server https://acme-staging.api.letsencrypt.org/ or the production server https://acme-v01.api.letsencrypt.org. Alternatively you can use another implemntation like Boulder or Pebble The jwk can be generated from a private key like this: jwk = &quot;priv/cert/selfsigned_key.pem&quot; |&gt; File.read!() |&gt; JOSE.JWK.from_pem() |&gt; JOSE.JWK.to_map()"},{"ref":"Certbot.Config.html#new/1","title":"Certbot.Config.new/1","type":"function","doc":""},{"ref":"Certbot.Config.html#t:t/0","title":"Certbot.Config.t/0","type":"type","doc":""},{"ref":"Certbot.Error.html","title":"Certbot.Error","type":"module","doc":"Struct to store errors"},{"ref":"Certbot.Error.html#from_struct/1","title":"Certbot.Error.from_struct/1","type":"function","doc":"Convert structs of the same shape, to a Certbot.Error struct. E.g. Acme.Error"},{"ref":"Certbot.Logger.html","title":"Certbot.Logger","type":"behaviour","doc":"Module and behaviour to log events"},{"ref":"Certbot.Logger.html#log/2","title":"Certbot.Logger.log/2","type":"function","doc":""},{"ref":"Certbot.Logger.html#c:log/2","title":"Certbot.Logger.log/2","type":"callback","doc":""},{"ref":"Certbot.Provider.html","title":"Certbot.Provider","type":"behaviour","doc":"Behaviour used for providing certficates by a given hostname."},{"ref":"Certbot.Provider.html#c:get_by_hostname/1","title":"Certbot.Provider.get_by_hostname/1","type":"callback","doc":""},{"ref":"Certbot.Provider.Acme.html","title":"Certbot.Provider.Acme","type":"module","doc":"Certificate provider for the Acme protocol When a request is made for a hostname, the provider will look into the certificate store (Certbot.CertificateStore) to see whether it has a certificate for that hostname. If so, it will return the certificate. If not, it will try to request a certificate using the acme client. This is done by retrieving an authorization, which has challenges. We need to prove to the acme server that we own the hostname. One of these challenges can be done over http. We use this one to prove ownership. The challenge is stored in the challenge store (Certbot.Acme.ChallengeStore), then the Acme server is asked to verify the challenge. The Certbot.Acme.Plug verifies the challenge by using the store. Next step is to build a Certificate Signing Request (csr) and send this to the Acme server. In the response there will be a url where the signed certificate can be retrieved from the Acme server. The downloaded certificate is used for the serving the request, and also stored in the certificate store for subsequent requests. Example use Certbot.Provider.Acme, acme_client: YourApp.Certbot, certificate_store: Certbot.CertificateStore.Default, challenge_store: Certbot.ChallengeStore.Default For the options that can be given to the use macro, see Certbot.Provider.Acme.Config"},{"ref":"Certbot.Provider.Acme.html#get_by_hostname/2","title":"Certbot.Provider.Acme.get_by_hostname/2","type":"function","doc":""},{"ref":"Certbot.Provider.Acme.Config.html","title":"Certbot.Provider.Acme.Config","type":"module","doc":"Configuration for the Certbot.Provider.Acme certificate provider. :acme_client -- Client implementing use Certbot, e.g. Myapp.Certbot :certificate_store -- Module used to store certificates, :challenge_store -- Module used to store certificates, :logger -- Module to log events, defaults to Certbot.Logger, :key_algorithm -- Algorithm used to generate keys for certificates, defaults to {:ec, :secp384r1}. Can also be e.g. {:rsa, 2048}"},{"ref":"Certbot.Provider.Acme.Config.html#new/1","title":"Certbot.Provider.Acme.Config.new/1","type":"function","doc":""},{"ref":"Certbot.Provider.Static.html","title":"Certbot.Provider.Static","type":"module","doc":"Static certificate provider Expects a certificates keyword with a map of hostnames as keys and %Certbot.Certificate{} structs as values. Also an example of a simple certificate provider. defmodule Myapp.StaticProvider do use Certbot.Provider.Acme, certificates: %{ &quot;example.com&quot; =&gt; %Certbot.Certificate{ cert: cert_der, key: {:RSAPrivateKey, key_der} } } end"},{"ref":"Certbot.Provider.Static.html#get_by_hostname/2","title":"Certbot.Provider.Static.get_by_hostname/2","type":"function","doc":""},{"ref":"Certbot.SSL.html","title":"Certbot.SSL","type":"module","doc":"Utility functions to deal with ssl and generating private keys"},{"ref":"Certbot.SSL.html#convert_private_key_to_der/1","title":"Certbot.SSL.convert_private_key_to_der/1","type":"function","doc":""},{"ref":"Certbot.SSL.html#convert_private_key_to_pem/1","title":"Certbot.SSL.convert_private_key_to_pem/1","type":"function","doc":""},{"ref":"Certbot.SSL.html#generate_csr/2","title":"Certbot.SSL.generate_csr/2","type":"function","doc":""},{"ref":"Certbot.SSL.html#generate_key/1","title":"Certbot.SSL.generate_key/1","type":"function","doc":""}]